// Copyright (C) 2013 S&P Capital IQ
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.spcapitaliq.platformexecuting

import java.io.{Closeable, File, InputStream, OutputStream}
import scala.annotation.tailrec
import scala.collection.{GenTraversable, GenTraversableLike}
import scala.collection.generic.CanBuildFrom

/** Internal utilities. */
private[platformexecuting] object Util {
  @inline def bracket[A, B](a: A)(f: A => B)(cl: A => Unit): B =
    try f(a) finally cl(a)

  final class OpenAs[A <: Closeable](val self: A) /* extends AnyVal */ {
    @inline final def openAs[B](f: A => B): B =
      bracket(self)(f)(_.close())
  }

  @inline implicit def OpenAs[A <: Closeable](a: A): OpenAs[A] = new OpenAs(a)

  /** `|@|` specialized for Scala collections. */
  def map2[A, AS, B, C, CS
         ](as: GenTraversableLike[A, AS], bs: GenTraversable[B]
         )(f: (A, B) => C)(implicit o: CanBuildFrom[AS, C, CS]): CS =
    for (a <- as; b <- bs) yield f(a, b)

  def copyIO(i: InputStream, o: OutputStream) {
    val b = Array.ofDim[Byte](1024)
    @tailrec def lp(): Unit = i.read(b) match {
      case r if r < 0 => ()
      case 0 => lp()
      case r => o.write(b, 0, r); lp()
    }
    lp()
  }
}
